

clear all;
clc
% 10.4.2013
% GENmodKQ.m  

% Robert Kollmann, ECARES, Universite Libre de Bruxelles & CEPR
% robert_kollmann@yahoo.com

% This program solves a simple closed economy RBC model to second order,
% and it applies a QUADRATIC KALMAN FILTER, and a PARTICLE FILTER to
% estimate the latent states of the model. 

global BETA CRRA ELS KPSI DEPRECss PSIss THss RHOth RHOpsi  
global omega omegaU

global DATA_KALMANQ  G_KALMANQ  


nS=7;   % number of states
nO=4;   % number of observables
nE=2;   % number of exogenous variables
nP2=.5*nS*(1+nS); % number of squares and cross-products of states
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

ssE=1;   % std of innov to exog variables
ssU=4;   % std of measurement error


omega=((ssE/100)^2)*eye(nE);   % Cov of innovations to exogenous var.
omegaU=((ssU/100)^2)*eye(nO);  % Cov of measurement error

% Selection matrix for observation equation
G=zeros(nO,nS); 
G(1:nO,1:nO)=eye(nO);
G_KALMANQ=G;

randn('seed',5001552); 


RESULT_STD_S2=[]; RESULT_STD_HP_S2=[];
RESULT_STD_S1=[]; RESULT_STD_HP_S1=[];


resRMSE_KQ=[]; resRMSE_PF=[]; resRMSE_LIN=[];
resMAX_KQ=[];  resMAX_PF=[];  resMAX_LIN=[];
resCORR_KQ=[]; resCORR_PF=[]; resCORR_LIN=[];


% ========== ITERATE OVER SIMULATED DATA SETS ==============
zksed=1; while zksed<=50;

display('=========================');    
tabulate1('zksed:  ',zksed)
zeit;
display('=========================');

F0a=randn(nS,1);  F0=((ssE/100)^2)*F0a/maxaa(F0a);
F1a=randn(nS,nS); F1=.99*F1a/maxaa(eig(F1a)); 

F2=randn(nS,nE);
F11=randn(nS,.5*nS*(nS+1));
F12=randn(nS,nS*nE);
F22=randn(nS,.5*nE*(nE+1));


% ========================================================================================
% ========================================================================================
%if zksed==1; PRED_MOM=1; end;
%if zksed>1;  PRED_MOM=0; end;
PRED_MOM=1;

if PRED_MOM==1;
    
% Predicted moments (unfiltered series)
[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omega);
% Predicted moments (HP filtered series)
[HP_VZtilde,HP_VZtilde1,HP_VS,HP_VS1]=gensys2_HPmoments(F0,F1,F2,F11,F12,F22,omega);

% MnHP1: Non-HP filtered series: (std with 2nd order approx)/(std with lin approx);
% MnHP2: Non-HP filtered series:  correlation between 1st & 2nd order approx;
MnHP1=sqrt(diag(VZtilde(1:nS,1:nS))./diag(VS));  
MnHP2=diag(VZtilde(1:nS,nS+nP2+1:end))./sqrt(diag(VZtilde(1:nS,1:nS)).*diag(VZtilde(nS+nP2+1:end,nS+nP2+1:end)));

% MHP1: HP filtered series: (std with 2nd order approx)/(std with lin approx);
% MHP2: HP filtered series:  correlation between 1st & 2nd order approx;
MHP1=sqrt(diag(HP_VZtilde(1:nS,1:nS))./diag(HP_VS));  
MHP2=diag(HP_VZtilde(1:nS,nS+nP2+1:end))./sqrt(diag(HP_VZtilde(1:nS,1:nS)).*diag(HP_VZtilde(nS+nP2+1:end,nS+nP2+1:end)));

send;
zeit;
display('KQ: quadr. Kalman;  LIN: Lin Kalman')
display('ssE%  ssU%') ; 
disp([num2str([ssE ssU],'%7.3f')]);
display('OBSERVABLES:');
tabulate1(sum(G)==1)
display('.....................');


display('relSTD: std(quadr approx)/std(lin approx); Corr: corr(Quadr,Lin)');
display('  Non-HP filt   HP filtered'); 
display('  relSTD Corr   relSTD Corr'); 
tabulate1([MnHP1 MnHP2  MHP1 MHP2]);

% ........................................................................
% MOMENTS GENERATED BY THE MODEL

resSTD_S2=[]; resSTD_HP_S2=[]; 
resSTD_S1=[]; resSTD_HP_S1=[]; 

i=0; while i<=rows(omega);
omegaFDF=zeros(size(omega));
if i==0; omegaFDF=omega; end;
if i>=1; omegaFDF(i,i)=omega(i,i); end;
[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omegaFDF);
[HP_VZtilde,HP_VZtilde1,HP_VS,HP_VS1]=gensys2_HPmoments(F0,F1,F2,F11,F12,F22,omegaFDF);
resSTD_S2=[resSTD_S2 sqrt(diag(VZtilde(1:nS,1:nS)))];
resSTD_HP_S2=[resSTD_HP_S2 sqrt(diag(HP_VZtilde(1:nS,1:nS)))];
resSTD_S1=[resSTD_S1 sqrt(diag(VS))];
resSTD_HP_S1=[resSTD_HP_S1 sqrt(diag(HP_VS))];
i=i+1; end;

RESULT_STD_S2=[RESULT_STD_S2;  resSTD_S2(:,1)'];
RESULT_STD_HP_S2=[RESULT_STD_HP_S2;  resSTD_HP_S2(:,1)'];
RESULT_STD_S1=[RESULT_STD_S1;  resSTD_S1(:,1)'];
RESULT_STD_HP_S1=[RESULT_STD_HP_S1;  resSTD_HP_S1(:,1)'];


display('.........................');
display('Std of S2: col 1: ALL shocks; cols 2-n: variance SHARES of indiv shocks')
display('... NOT FILTERED ...');
tabulate1([resSTD_S2(:,1)    (resSTD_S2(:,2:end)./(resSTD_S2(:,1)*ones(1,nE))).^2]);
display('... HP FILTERED ...');
tabulate1([resSTD_HP_S2(:,1) (resSTD_HP_S2(:,2:end)./(resSTD_HP_S2(:,1)*ones(1,nE))).^2]);
space(1);

%error('AAA');

display('.........................');
display('Std of S2: col 1: ALL shocks; cols 2-n: Std with just indiv shocks')
display('... NOT FILTERED ...');
tabulate1(resSTD_S2);
display('... HP FILTERED ...');
tabulate1(resSTD_HP_S2);

display('.........................');
display('Std of S1: col 1: ALL shocks; cols 2-n: Std with just indiv shocks')
display('... NOT FILTERED ...');
tabulate1(resSTD_S1);
display('... HP FILTERED ...');
tabulate1(resSTD_HP_S1);


% ======================================================
% IMPULSE RESPONSES
IMPRES=0;
if IMPRES==1;
    
i=1; while i<=cols(omega);
exo=zeros(201,cols(omega));
exo(1,i)=.1;
%Pwsl1=arsimqL_P(EZtilde(1:nS),exo,FD,FDD,M11,M2,pick,zu,sig);
%Pwsl0=arsimqL_P(EZtilde(1:nS),0*exo,FD,FDD,M11,M2,pick,zu,sig);
%Pwsl=(Pwsl1(2:end,1:nS)-Pwsl0(2:end,1:nS));
[Pwsl1,S1wsl1,S1Pwsl1]=arsimqL_STACK(EZtilde(1:nS),zeros(nS,1),exo,F0,F1,F2,F11,F12,F22);
[Pwsl0,S1wsl0,S1Pwsl0]=arsimqL_STACK(EZtilde(1:nS),zeros(nS,1),0*exo,F0,F1,F2,F11,F12,F22);
Pwsl=(Pwsl1(:,1:nS)-Pwsl0(:,1:nS));

display('......................................');
horiz=['  0 ';'  1 ';'  2 ';'  4 ';' 10 ';' 20 ';'100 ';'200 '];
display('QUADRATIC IMPULSE RESPONSE');
tabulate1(horiz,Pwsl([1;2;3;5;11;21;101;201],:))
display('.........');
display('LINEAR IMPULSE RESPONSE');
tabulate1(horiz,S1wsl1([1;2;3;5;11;21;101;201],:))
space(2);
i=i+1; end;

end;  % end if IMPRES==1;

fin;

end;   % end if PRED_MOM==1
% ========================================================================================
% ========================================================================================

% error('AAA');


% .........................................................................
% ...........................SIMULATED DATA ................................

%T=1000;
%T=1000;
T=100;


%T=100000;
%T=2000;
e=randn(T,cols(omega))*cholPSD(omega);
u=randn(T,nO)*cholPSD(omegaU);
z0=zeros(nS,1);

[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omega); 
S1init=zeros(nS,1);
S2init=EZtilde(1:nS,:);
[S2,S1,S1P]=arsimqL_STACK(S2init,S1init,e,F0,F1,F2,F11,F12,F22);
% e=[e(1)';e(2)';...;e(T)'];
% S2: second-order accurate simulated series: [S2(1)';S2(2)';...;S2(T)']
% S1: first-order accurate simulated series:  [S1(1)';S1(2)';...;S1(T)']
% S1P: one-period ahead predictions of 1st-order accurate series:
% [E{S1(2)|I(1)}';E{S1(3)|I(2)}';...;E{S1(T+1)|I(T)}']

%S2_Z=arsimqL_P(S2init,e,FD,FDD,M11,M2,pick,zu,sig);
%if maxaa(S2_Z(2:end,1:nS)-S2)>1e-8; error('AAA'); end;

%S1init=zeros(nS,1);
%S1=arsim(S1init,e*F2',F1);
%S1i=[S1init';S1(1:end-1,:)]; 
%S1P=S1*F1';  % E(S1(t+1)|I(t)

%S2init=EZtilde(1:nS,:);
%e2=ones(T,1)*F0'+e*F2'+polynom2(S1i)*F11'+ ...
%  polynom2ALL(S1i,e)*F12'+polynom2(e)*F22';
%S2=arsim(S2init,e2,F1);
%S2i=[S1init';S2(1:end-1,:)];

tabulate1(std(S2)./std(S1))
tabulate1(correl2(S1,S2))

% ............................................

%z0=zeros(rows(xss1),1); w2=arsimqL(z0,e);
%if maxaa(S2-w2(2:end,1:8))>1e-8; error('AA'); end;

% OBSERVATION EQUATION
y=S2*G'+u;   

%y2=polynom2(y);
%yC=S1*G'+u;   
%y2a=polynom2(S1*G'+u);   


DATA=y;
DATA_KALMANQ=DATA;



[resOBS,resERR,resEZ,Log_Likelihood]=KALMANQ_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,y);
[resOBS_LIN,resERR_LIN,resEZ_LIN,Log_Likelihood_LIN]=KALMAN_LIN_FILTER(F1,F2,G,omega,omegaU,demean(y));
[resOBS_PF,resERR_PF,resEZ_PF,resNeff,Log_Likelihood_PF,PT]=PARTICLE_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,100000,y);
% Apply Linear Kalman filter to DEMEANED observables !
%[resOBS2,resERR2,resEZ2,Log_Likelihood2]=KALMANQ_2_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,y);


space(1)


resRMSE_KQ=[resRMSE_KQ;   rmse(resEZ(:)-S2(:))     rmse(resEZ-S2)];
resRMSE_PF=[resRMSE_PF;   rmse(resEZ_PF(:)-S2(:))  rmse(resEZ_PF-S2)];
resRMSE_LIN=[resRMSE_LIN; rmse(resEZ_LIN(:)-S2(:)) rmse(resEZ_LIN-S2)];

resMAX_KQ=[resMAX_KQ;   maxABS(resEZ(:)-S2(:))     maxABS(resEZ-S2)];
resMAX_PF=[resMAX_PF;   maxABS(resEZ_PF(:)-S2(:))  maxABS(resEZ_PF-S2)];
resMAX_LIN=[resMAX_LIN; maxABS(resEZ_LIN(:)-S2(:)) maxABS(resEZ_LIN-S2)];

resCORR_KQ=[resCORR_KQ;   correl2(resEZ,S2)];
resCORR_PF=[resCORR_PF;   correl2(resEZ_PF,S2)];
resCORR_LIN=[resCORR_LIN; correl2(resEZ_LIN,S2)];


display('RMSE for latent states| (1): KQ; (2): PF;  (3): LIN ');
tabulate1([100*rmse(resEZ(:)-S2(:)) 100*rmse(resEZ-S2); ...
           100*rmse(resEZ_PF(:)-S2(:)) 100*rmse(resEZ_PF-S2); ...
           100*rmse(resEZ_LIN(:)-S2(:)) 100*rmse(resEZ_LIN-S2)])
     
space(1);
display('Max estimation error for latent states');
tabulate1([maxABS(resEZ(:)-S2(:)) maxABS(resEZ-S2); ...
           maxABS(resEZ_PF(:)-S2(:)) maxABS(resEZ_PF-S2); ...
           maxABS(resEZ_LIN(:)-S2(:)) maxABS(resEZ_LIN-S2)])

space(1);
display('Corr. between estimate and true state')
ASZ=['       ';'       ';'       '];
tabulate1(ASZ,[correl2(resEZ,S2); correl2(resEZ_PF,S2); correl2(resEZ_LIN,S2)])

save resZKSED resRMSE_KQ resRMSE_PF resRMSE_LIN resMAX_KQ resMAX_PF resMAX_LIN resCORR_KQ resCORR_PF resCORR_LIN;


zksed=zksed+1; end;



% ..................................................................................
% ..................................................................................

send;
%xy9(resEZ,resEZ_PF,S2)

zeit;
display('Prediction errors: state vars. KQ: quadr. Kalman, PF: particle filt, LIN: Lin Kalman')
display('ssE%   sU%') ; 
disp([num2str([ssE ssU],'%7.3f')]);

disp(['T Particles NrReplics   ',[num2str([T PT rows(resRMSE_KQ)],'%10.0f')]]);


display('OBSERVABLES:');
tabulate1(sum(G)==1)
display('.....................');

CVP=['KQ  ';'PF  ';'LIN '];
display(['      ALL   |','INDIVIDUAL VARIABLES'])
display('MEAN RMSEs across simulation runs');
tabulate1(CVP,[mean(resRMSE_KQ);mean(resRMSE_PF);mean(resRMSE_LIN)]);
display('MEDIAN RMSEs across simulation runs');
tabulate1(CVP,[median(resRMSE_KQ);median(resRMSE_PF);median(resRMSE_LIN)]);
display('MAX RMSEs across simulation runs');
tabulate1(CVP,[max(resRMSE_KQ);max(resRMSE_PF);max(resRMSE_LIN)]);
display('MIN RMSEs across simulation runs');
tabulate1(CVP,[min(resRMSE_KQ);min(resRMSE_PF);min(resRMSE_LIN)]);
display('MIN %RMSEs across simulation runs');
tabulate1(CVP,100*[min(resRMSE_KQ);min(resRMSE_PF);min(resRMSE_LIN)]);
display('fraction of RMSE smaller under KQ (across simulation runs)');
tabulate1(CVP([2 3],:),[mean(resRMSE_KQ<resRMSE_PF);mean(resRMSE_KQ<resRMSE_LIN)]);
space(1);
display(['      ALL   |','INDIVIDUAL VARIABLES'])
display('MEAN MAX errors across simulation runs');
tabulate1(CVP,[mean(resMAX_KQ);mean(resMAX_PF);mean(resMAX_LIN)]);
display('MEDIAN errors across simulation runs');
tabulate1(CVP,[median(resMAX_KQ);median(resMAX_PF);median(resMAX_LIN)]);
display('MAX of MAX errors across simulation runs');
tabulate1(CVP,[max(resMAX_KQ);max(resMAX_PF);max(resMAX_LIN)]);
display('fraction of MAX errors smaller under KQ (across simulation runs)');
tabulate1(CVP([2 3],:),[mean(resMAX_KQ<resMAX_PF);mean(resMAX_KQ<resMAX_LIN)]);
space(1);
display(['      ALL   |','INDIVIDUAL VARIABLES'])
display('MEAN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[mean(resCORR_KQ);mean(resCORR_PF);mean(resCORR_LIN)]);
display('MEDIAN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[median(resCORR_KQ);median(resCORR_PF);median(resCORR_LIN)]);
display('MIN Correlations between filtered and true states');
tabulate1([CVP  repmat(['       '],3,1)],[min(resCORR_KQ);min(resCORR_PF);min(resCORR_LIN)]);


% .......................................................................................
% ............................ MEAN & MEDIAN MOMENTS.....................................
KRA=repmat(['           '],4,1);
display('MEAN STDs:   (1) Std(S2);  (2) Std(S1);  (3) Std(HPS(S2));  (4) Std(HPS(S1)) ');
tabulate1(KRA,[mean(RESULT_STD_S2); mean(RESULT_STD_S1); mean(RESULT_STD_HP_S2); mean(RESULT_STD_HP_S1)]);
display('MEDIAN STDs: (1) Std(S2);  (2) Std(S1);  (3) Std(HPS(S2));  (4) Std(HPS(S1)) ');
tabulate1(KRA,[median(RESULT_STD_S2); median(RESULT_STD_S1); median(RESULT_STD_HP_S2); median(RESULT_STD_HP_S1)]);
display('MAX STDs:    (1) Std(S2);  (2) Std(S1);  (3) Std(HPS(S2));  (4) Std(HPS(S1)) ');
tabulate1(KRA,[max(RESULT_STD_S2); max(RESULT_STD_S1); max(RESULT_STD_HP_S2); max(RESULT_STD_HP_S1)]);
display('MIN %STDs:    (1) Std(S2);  (2) Std(S1);  (3) Std(HPS(S2));  (4) Std(HPS(S1)) ');
tabulate1(KRA,100*[min(RESULT_STD_S2); min(RESULT_STD_S1); min(RESULT_STD_HP_S2); min(RESULT_STD_HP_S1)]);


fin;



