

function [resOBS_LIN,resERR_LIN,resEZ_LIN,Log_Likelihood_LIN]=KALMAN_LIN_FILTER(F1,F2,G,omega,omegaU,DATA);

% March 4, 2013
% Robert Kollmann, robert_kollmann@yahoo.com
% This proc takes the coefficients of a 2nd order DSGE model approximation, and the DATA as inputs. 
% The proc returns:
% (a) resOBS: date t estimate of true values of date t observables
% (b) resERR: date t prediction error: y(t)=E(y*t)|I(t-1)  
% (c) resEZ: date t estimate of true 2nd order state: E(S2(t)|I(t)) 

% February 18, 2013
% Robert Kollmann, robert_kollmann@yahoo.com

if nargin~=6; error('QQQ'); end;

nS=rows(F1);    % number of states
nE=rows(omega); % number of exogenous innovations
nO=rows(G);     % number of observables
nP2=.5*nS*(nS+1); 
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

if rows(F1)~=nS; error('AA1a'); end;
if cols(F1)~=nS; error('AA1b'); end;
if rows(F2)~=nS; error('AA2a'); end;
if cols(F2)~=nE;  error('AA2b'); end;
if cols(omega)~=nE; error('AA7'); end;
if cols(DATA)~=nO;  error('AA8a'); end;
if rows(DATA)==0;   error('AA8b'); end;
if cols(G)~=nS;     error('AA9'); end;
if rows(omegaU)~=cols(omegaU); error('AA13'); end;
if rows(G)~=nO;     error('AA14a'); end;
if cols(G)~=nS;     error('AA14b'); end;

if maxaa(eig(F1))>=.999; res=-1e40; return; end;

y=DATA;


VS1=ardouble(F1,F2*omega*F2');  % E(S1*S1')
if maxaa(VS1-(F1*VS1*F1'+F2*omega*F2'))>1e-8; error('AAA'); end;
ES1=zeros(nS,1);


VS=VS1;   % Var(Ztilde(0)|I(0))
ES=ES1;   %   E(Ztilde(0)|I(0)


resOBS=[]; resERR=[]; resEZ=[]; LOG_L=[];

t=1; while t<=rows(y);
if (t/5000-ceil(t/5000))==0; display(t);  end; 

VSp=F1*VS*F1'+F2*omega*F2'; % Var(S1(t)|I(t-1))
kpsi=inv(G*VSp*G'+omegaU);           
psi=VSp*G'*kpsi;

ESp=F1*ES;                          % E(S1(t)|I(t-1))
ESpp=ESp+VSp*G'*kpsi*(y(t,:)'-G*ESp);  % E(Ztilde(t)|I(t))

VSppOLD=VSp-VSp*G'*kpsi*G*VSp';         % Var(Ztilde(t)|I(t))

RK=eye(rows(VSp))-psi*G;
VSpp=RK*VSp*RK'+psi*omegaU*psi';

if maxaa(VSpp-VSppOLD)>1e-8; error('AA'); end;


resOBS=[resOBS; (G*ESpp)'];        % date t estimate of true values of date t observables
resERR=[resERR; (y(t,:)'-G*ESp)']; % date t prediction error: y(t)=E(y*t)|I(t-1)  
resEZ=[resEZ; ESpp(1:nS)'];             % E(S2(t)|I(t)) 

VS=VSpp;
ES=ESpp;

logh=.5*log(det(kpsi))-.5*(y(t,:)'-G*ESp)'*kpsi*(y(t,:)'-G*ESp)-.5*nO*log(2*pi);
LOG_L=[LOG_L; logh];

t=t+1; end;


resOBS_LIN=resOBS;
resERR_LIN=resERR;
resEZ_LIN=resEZ;


Log_Likelihood_LIN=sum(LOG_L);

