

function [resOBS,resERR,resEZ,resNeff,Log_Likelihood,PT]=PARTICLE_FILTER(F0,F1,F2,F11,F12,F22,G,omega,omegaU,PT,DATA);
% Robert Kollmann, robert_kollmann@yahoo.com

if nargin~=11; error('QQQ'); end;
if nargout~=6; error('QQQ'); end;

% March 4, 2013
% This proc takes the coefficients of a 2nd order 'pruned' DSGE model approximation, 
%  the Number of Particles (PT), and the (Txn) DATA matrix as inputs. 

% Underlying model: 
% S2(t+1)=F0+F1*S2(t)+F2*e(t+1)+F11*polynom2(S1(t))+F12*polynom2ALL(S1(t),e(t+1))+F22*polynom2(e(t+1))
% S1(t+1)=F1*S1(t)+F2*e(t+1)

% S2(t+1): second-order accurate model solution
% S1(t+1): first-order accurate model solution

% Recursion for polynom2(S1(t+1)): 
%  polynom2(S1(t+1))=K11*polynom2(S1(t))+K12*polynom2ALL(S1(t),e(t+1))+K22*polynom2(e(t+1))

% A state-space representation is constructed in terms of 
% Ztilde(t+1)=[S2(t+1);polynom2(S1(t+1));S1(t+1)]
% Ztilde(t+1)=G0+G1*Ztilde(t)+G2*e(t+1)+G12*polynom2ALL(S(t),e(t+1))+G22*polynom2(e(t+1))

% Observation equation:  y(t)=G*S2(t)+u(t), with u(t): i.i.d. measurement  error

% The proc uses the standard PARTICLE FILTER to construct estimate of S2(t)
% See An & Schorfheide (2007), pp.159-160. 

% The proc returns:
% (a) resOBS: date t estimate of true values of date t observables
% (b) resERR: date t prediction error: y(t)-E(y(t)|I(t-1)  
% (c) resEZ: date t estimate of true 2nd order state: E(S2(t)|I(t)) 
% (d) resNeff: number of effective particles
% (r) Log_Likelihood:  Log likelihood, based on Particle Filter


nS=rows(F0);    % number of states
nE=rows(omega); % number of exogenous innovations
nO=rows(G);     % number of observables
nP2=.5*nS*(nS+1); 
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

if rows(F1)~=nS; error('AA1'); end;
if rows(F2)~=nS; error('AA2'); end;
if rows(F11)~=nS; error('AA3'); end;
if rows(F12)~=nS; error('AA4'); end;
if rows(F22)~=nS; error('AA5'); end;
if cols(F2)~=nE;  error('AA6'); end;
if cols(omega)~=nE; error('AA7'); end;
if cols(DATA)~=nO;  error('AA8a'); end;
if rows(DATA)==0;   error('AA8b'); end;
if cols(G)~=nS;     error('AA9'); end;
if cols(F11)~=nP2;  error('AA10'); end;
if cols(F12)~=nE*nS;  error('AA11'); end;
if cols(F22)~=.5*nE*(nE+1);  error('AA12'); end;
if rows(omegaU)~=cols(omegaU); error('AA13'); end;
if rows(G)~=nO;     error('AA14a'); end;
if cols(G)~=nS;     error('AA14b'); end;
if rows(PT)~=1;     error('AA15a'); end;
if cols(PT)~=1;     error('AA15b'); end;
if maxaa(PT/ceil(PT)-1)>1e-10;  error('AA16'); end;


y=DATA;

[VZtilde,VZtilde1,EZtilde,VS,VS1]=gensys2_moments(F0,F1,F2,F11,F12,F22,omega); 
ES2=EZtilde(1:nS);
VS2=VZtilde(1:nS,1:nS);

%PT=100000;  % number of particles
%PT=50000;   

R_VS2=cholPSD(VS2);
if maxaa(R_VS2'*R_VS2-VS2)>1e-7; error('AA'); end;

%S2part=ES2*ones(1,PT)+R_VS2'*randn(nS,PT);
%S1part=cholPSD(VS)'*randn(nS,PT);

SpartE=EZtilde*ones(1,PT)+cholPSD(VZtilde)'*randn(nZ,PT);
S2part=SpartE(1:nS,:);
S1part=SpartE(nS+nP2+1:end,:);

%X=[S2part;polynom2(S1part')';S1part];
%xy8(cov(X'),VZtilde)
%xy9(cov(X'),VZtilde,cov(SpartE'))


% ........................................................
resES2=[]; resES1=[]; resWp=[]; 
resNeff=[]; resLOGL=[];


t=1; while t<=rows(DATA);
if t/50==ceil(t/50); display([t rows(DATA)]); end;

eP=cholPSD(omega)*randn(nE,PT); 

S2P=F0*ones(1,PT)+F1*S2part+F2*eP+F11*polynom2(S1part')'+F12*polynom2FULL(S1part',eP')'+F22*polynom2(eP')';
S1P=F1*S1part+F2*eP; 



ERRy=y(t,:)'*ones(1,PT)-G*S2P;
LNp=logmvnpdf(ERRy',zeros(1,nO),omegaU);
%ELNp=0*LNp; ELNp(LNp>-740)=exp(LNp(LNp>-740));
%Wp1=ELNp./sum(ELNp);

%Wp=exp(LNp-max(LNp)+700)/sum(exp(LNp-max(LNp)+700));
%ELNp=exp(LNp-max(LNp)+700)*exp(max(LNp)-700);

% April 8, 2013: BETTER FORMULA FOR weights !
%Wp=exp(LNp-max(LNp)+708)/sum(exp(LNp-max(LNp)+708));
BXLp=exp(LNp-max(LNp)+708);
Wp=(BXLp/max(BXLp))/sum(BXLp/max(BXLp));

ELNp=exp(LNp-max(LNp)+708)*exp(max(LNp)-708);



%LNpP=LNp+500;
%ELNp=0*LNpP; ELNp(LNpP>-740)=exp(LNpP(LNpP>-740))*exp(-500);
%Wp=ELNp./sum(ELNp);

%y = randsample(n,k,true,w) or y = randsample(population,k,true,w) 
%returns a weighted sample taken with replacement, using a vector of positive weights w, 
%whose length is n. The probability that the integer i is selected for an entry of y is w(i)/sum(w). 
%Usually, w is a vector of probabilities. 
%randsample does not support weighted sampling without replacement.

Cr=randsample(PT,PT,true,Wp');

S2part=S2P(:,Cr);
S1part=S1P(:,Cr);

resES2=[resES2; Wp'*S2part'];
resES1=[resES1; Wp'*S1part'];
%resWp=[resWp; Wp']; 
resNeff=[resNeff; 1/sum(Wp.^2)];
resLOGL=[resLOGL; mean(ELNp)];

t=t+1; end;

zeit;

resEZ=resES2;    % resEZ: date t estimate of true 2nd order state: E(S2(t)|I(t))
resOBS=resEZ*G'; % resOBS: date t estimate of true values of date t observables
resERR=y-resOBS; % resERR: date t prediction error: y(t)-E(y(t)|I(t-1)  
Log_Likelihood=sum(resLOGL);  % Log likelihood, based on Particle Filter



