

function [S2,S1,S1P]=arsimqL_STACK(S2init,S1init,exo,F0,F1,F2,F11,F12,F22);

% March 7, 2013. 
% Robert Kollmann, robert_kollmann@yahoo.com
% Proc for simulating the 2nd order accurate model solution
% Format:  
% S2(t): second-order accurate solution
% S1(t): first-order accurate solution
% S2(t+1)=F0+F1*S2(t)+F2*e(t+1)+F11*polynom2(S1(t))+ ... 
%   F12*polynom2ALL(S1(t),e(t+1))+ F22*polynom2(e(t+1));
% S1(t+1)=F1*S1(t)+F2*e(t+1);
% 

% INPUTS:
% S2init: S2(0), initial value for 2nd order accurate variables
% S1init: S1(1), initial value for 1st order accurate variables
% exo: innovations, [e(1)';e(2)';...;e(T)']
% other inputs: coefficients of 2nd order model solution computed by gensys2 
% 
% 
% OUTPUTS: 
% S2: second-order accurate simulated series: [S2(1)';S2(2)';...;S2(T)']
% S1: first-order accurate simulated series:  [S1(1)';S1(2)';...;S1(T)']
% S1P: one-period ahead predictions of 1st-order accurate series:
% [E{S1(2)|I(1)}';E{S1(3)|I(2)}';...;E{S1(T+1)|I(T)}']


if nargin~=9; error('WWW'); end;

%[F0,F1,F2,F11,F12,F22]=gensys2_TO_KALMANQ(FD,FDD,M11,M2,pick,zu,sig);

nS=rows(F0);    % number of states
nE=cols(F2); % number of exogenous innovations
%nO=rows(G);     % number of observables
nP2=.5*nS*(nS+1); 
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

if rows(F1)~=nS; error('AA1'); end;
if rows(F2)~=nS; error('AA2'); end;
if rows(F11)~=nS; error('AA3'); end;
if rows(F12)~=nS; error('AA4'); end;
if rows(F22)~=nS; error('AA5'); end;
%if cols(F2)~=nE;  error('AA6'); end;
%if cols(omega)~=nE; error('AA7'); end;
if cols(F11)~=nP2;  error('AA10'); end;
if cols(F12)~=nE*nS;  error('AA11'); end;
if cols(F22)~=.5*nE*(nE+1);  error('AA12'); end;

if rows(S2init)~=nS; error('BB1'); end;
if cols(S2init)~=1; error('BB2'); end;
if rows(S1init)~=nS; error('CC1'); end;
if cols(S1init)~=1; error('CC2'); end;

    
S1init=zeros(nS,1);
S1=arsim(S1init,exo*F2',F1);

S1i=[S1init';S1(1:end-1,:)]; 
S1P=S1*F1';  % E(S1(t+1)|I(t)

e2=ones(rows(exo),1)*F0'+exo*F2'+polynom2(S1i)*F11'+ ...
  polynom2ALL(S1i,exo)*F12'+polynom2(exo)*F22';
S2=arsim(S2init,e2,F1);
%S2i=[z0';S2(1:end-1,:)];


