


function [HP_VZtilde,HP_VZtilde1,HP_VS,HP_VS1]=gensys2_HPmoments(F0,F1,F2,F11,F12,F22,omega);
% March 4, 2013
% Robert Kollmann, robert_kollmann@yahoo.com
% This proc takes the coefficients of a 2nd order DSGE model approximation
% computed by gensys, and it reports moments of HP filtered variables

% S2(t): second-order accurate model solution
% S1(t): first-order accurate model solution
% Ztildehp(t)=HP([S2(t);polynom2(S1(t));S1(t)],LAM);
% S1hp(t)=HP(S1(t),LAM);
% 
% HP_VZtilde: cov(Ztildehp);
% HP_VZtilde1=E{(Ztildehp(t)-E(Ztildehp(t))*(Ztildehp(t-1)-E(Ztildehp(t-1))}
% HP_VS=E{S1hp(t)*S1hp(t)'}
% HP_VS1=E{S1hp(t)*S1hp(t-1)'}

% February 18, 2013
% Robert Kollmann, robert_kollmann@yahoo.com

if nargin~=7; error('AAA'); end;

%[F0,F1,F2,F11,F12,F22]=gensys2_TO_KALMANQ(FD,FDD,M11,M2,pick,zu,sig);

nS=rows(F0);    % number of states
nE=rows(omega); % number of exogenous innovations
%nO=rows(G);     % number of observables
nP2=.5*nS*(nS+1); 
nZ=nS+nP2+nS;  % number of elements of Z=[S1; P2(S1); S1]

if rows(F1)~=nS; error('AA1'); end;
if rows(F2)~=nS; error('AA2'); end;
if rows(F11)~=nS; error('AA3'); end;
if rows(F12)~=nS; error('AA4'); end;
if rows(F22)~=nS; error('AA5'); end;
if cols(F2)~=nE;  error('AA6'); end;
if cols(omega)~=nE; error('AA7'); end;
%if cols(DATA)~=nO;  error('AA8a'); end;
%if rows(DATA)==0;   error('AA8b'); end;
%if cols(G)~=nS;     error('AA9'); end;
if cols(F11)~=nP2;  error('AA10'); end;
if cols(F12)~=nE*nS;  error('AA11'); end;
if cols(F22)~=.5*nE*(nE+1);  error('AA12'); end;
%if rows(omegaU)~=cols(omegaU); error('AA13'); end;
%if rows(G)~=nO;     error('AA14a'); end;
%if cols(G)~=nS;     error('AA14b'); end;

if maxaa(eig(F1))>=.999; res=-1e40; return; end;


%Gtilde=[G zeros(nO,nP2+nS)];




% ..........................................................................
% ..........................................................................
% QUADRATIC KALMAN FILTER

P2nS=[];
k=0;
i=1; while i<=nS;
j=1; while j<=nS;
k=k+1;    
if j>=i; P2nS=[P2nS;k]; end;
j=j+1; end;
i=i+1; end;


P2nE=[];
k=0;
i=1; while i<=nE;
j=1; while j<=nE;
k=k+1;    
if j>=i; P2nE=[P2nE;k]; end;
j=j+1; end;
i=i+1; end;

% is A is nSxnS matrix, then A(P2nS) produces a column vector consisting of
% the elements of A on and below the main diagonal


% recursion for P2(S1(t))
K11=[];  K12=[]; K22=[];
i=1; while i<=nS;
j=i; while j<=nS;
WSL=F1(i,:)'*F1(j,:);
WSP=WSL+WSL'-diag(diag(WSL));
K11=[K11; WSP(P2nS)'];
% ...........
DEH=F1(i,:)'*F2(j,:)+F1(j,:)'*F2(i,:);
DEHa=DEH'; K12=[K12; DEHa(:)'];
% ...........
BRE=F2(i,:)'*F2(j,:);
KLE=BRE+BRE'-diag(diag(BRE));
K22=[K22; KLE(P2nE)'];
j=j+1; end;
i=i+1; end;


% The recursion for Z(t)=[S2(t);polynom2(S1(t));S1(t)]
G0=[F0;zeros(nP2+nS,1)];
G1=zeros(nZ,nZ);
G1(1:nS,1:nS)=F1; G1(1:nS,nS+1:nS+nP2)=F11; 
G1(nS+1:nS+nP2,nS+1:nS+nP2)=K11;
G1(nS+nP2+1:end,nS+nP2+1:end)=F1;
G2=[F2;zeros(nP2,nE);F2];
G12=[F12;K12;zeros(nS,nS*nE)];
G22=[F22;K22;zeros(nS,.5*nE*(nE+1))];

%tabulate1(G22)
%space(1)

% .............................................................
% covariance of polynom2(e): VP2e
resI=[];
i=1; while i<=nE;
j=i; while j<=nE;
resI=[resI; [i j]];
j=j+1; end;
i=i+1; end;


%sA=(1:1:nE)'*ones(1,nE); 
%sB=ones(nE,1)*(1:1:nE); 
%resIa=[sB(P2nE) sA(P2nE)];
%if maxaa(resIa-resI)>1e-8; error('AA');


VP2e_A=zeros(.5*nE*(nE+1),.5*nE*(nE+1));
i=1; while i<=.5*nE*(nE+1);
j=i; while j<=.5*nE*(nE+1);
r1=resI(i,1); r2=resI(i,2);
s1=resI(j,1); s2=resI(j,2);
fr1=zeros(nE,1); fr1(r1)=1; 
fr2=zeros(nE,1); fr2(r2)=1; 
fs1=zeros(nE,1); fs1(s1)=1; 
fs2=zeros(nE,1); fs2(s2)=1; 
VP2e_A(i,j)=2*trace(.5*((fr1*fr2')+(fr2*fr1'))*omega*.5*((fs1*fs2')+(fs2*fs1'))*omega);
j=j+1; end;
i=i+1; end;
VP2e=(VP2e_A+VP2e_A')-diag(diag(VP2e_A));

% Mean of polynom2(e)
EP2e=omega(P2nE);

% .......................................................................
% VS: unconditional covariance of S1(t)
VS=ardouble(F1,F2*omega*F2');  % E(S1*S1')
if maxaa(VS-(F1*VS*F1'+F2*omega*F2'))>1e-6; 
    save klm7; res=-1e40; return; end;
VS1=F1*VS;  % E{S1(t)*S1(t-1)'}

% VZtilde: unconditional covariance of  Z(t)=[S2(t);polynom2(S1(t));S1(t)]
VZtilde=ardouble(G1,G2*omega*G2'+G12*kron(VS,omega)*G12'+G22*VP2e*G22'); % Var(Ztilde)
%if maxaa(VZtilde-(G1*VZtilde*G1'+G2*omega*G2'+G12*kron(VS,omega)*G12'+G22*VP2e*G22'))>1e-6; 
%    save klm8; res=-1e40; return; end;

%VZtilde1=G1*VZtilde;  %E{(Z(t)-E(Z(t))*(Z(t-1)-E(Z(t-1))}
%EZtilde=inv(eye(nZ)-G1)*(G0+G22*omega(P2nE));   %  E(Ztilde)


% MOMENTS OF HP FILTERED SERIES: 
% HP_VS=E(S1hp(t)*S1hp(t)');  HP_VS1=E(S1hp(t)*S1hp(t-1)');
% HP_VZtilde=E(Ztildehp(t)*Ztildehp(t)');  HP_VZtilde1=E(Ztildehp(t)*Ztildehp(t-1)')

LAM=1600;  
[HP_VS,HP_VS1]=hpm9(F1,F2*omega*F2',LAM);
[HP_VZtilde,HP_VZtilde1]=hpm9(G1,G2*omega*G2'+G12*kron(VS,omega)*G12'+G22*VP2e*G22',LAM);



