

function [s,s1] = hpm9(r,v,LAM); 

% 3.6.2008 THIS USES A LARGER NUMBER OF TERMS than hpm9old.m !!!!
% Robert Kollmann, robert_kollmann@yahoo.com

% 14.5.2008
% THIS PROC COMPUTES MOMENTS FOR HP FILTERED VAR(1) PROCESS, 
% WHERE THE SMOOTHING OF THE HP FILTER IS SET AT  'LAM'

% THE PROC USES THE EXACT WEIGHTS OF THE HP FILTER DERIVED BY 
%   Tucker McElroy, 2008, 'Exact formulas for the Hodrick-Prescott filter', 
%   Econometrics JOurnal, Vol. 11, pp. 209-217
%     see equations (2.12), (2.14), (2.15) and (3.4)

%  HP-FILTERED COV-MATRIX FOR VAR(1) PROCESS
%  @ June 7, 1994. C:\RBC\HPM7.M
%  USE THIS TO COMPUTE HP-filtered Cov-matrix
%  of an AR(1): x(t)=r*x(t-1)+e(t), E[e(t)*e(t)']=v;
%  Let x'(t) be HP filtered variable.
%  This program returns 2 variables:
%  E[x(t)*x'(t)] and E[x(t)*x'(t-1)].

%  THIS PROGRAM SEEMS TO GIVE BEST RESULTS.
%  This applies "Post-War US Business Cycles: an Empirical Investigation"
%  by Hodrick, R. & E. Prescott (1981), p.9.
%  The HP filter is written as a two-sided, symmetric moving average filter,
%  with weights given by h(i)=1-w(i), where w(i) is defined on p.9
%  of H&P.
%  Idea: let x'(t) denote HP-filtered x(t), can write
%  x'(t)=K(0)e(t)+ K(1)e(t-1) + K(2)e(t-2) +....
%                + K(-1)e(t+1)+ K(-2)e(t+2)+.... .
%  Express K(i) in terms of r and h(j)'s.
%  N.B.:  K(j)=K(j-1)*r+h(j)       ($)

%  This program computes K(-200), and obtains K(-199), K(-198),..,K(-1),K(0),K(1),
%  K(2),..,K(200) iteratively, using ($).


if rows(r)~=cols(r); error('Q1'); end;
if rows(v)~=cols(v); error('Q2'); end;
if rows(r)~=rows(v); error('Q3'); end;
if rows(LAM)~=1;    error('Q4'); end;
if cols(LAM)~=1;    error('Q4'); end;
if LAM<=0; error('LAM has to be strictly positive'); end;

hi=eye(rows(v));

s=0; zk=0;
j=-200;    % ZAR=200; j=-ZAR
k=0; 
a=eye(rows(r));

q=0; while q<=400;    
  k=k+hz(j-q,LAM)*a;
  a=a*r; 
q=q+1; end;

% k: K(-200);  k: K(-ZAR)

s=s+k*v*k';
zk=zk+hz(200-1,LAM)*v*k';   %  zk=zk+hz(ZAR-1,LAM)*v*k';

% cov terms in e(t+199), e(t+198),...e(0).@
j=199; while j>=0;   % j=ZAR-1; while j>=0
k=k*r+hi*hz(j,LAM);
s=s+k*v*k';
zk=zk+hz(j-1,LAM)*v*k';
j=j-1; end;


% @ cov terms in e(t-1), e(t-2), e(t-3),..,e(t-200).@
res=k;
j=1; while j<=200;
res=res*r+hi*hz(j,LAM);
s=s+res*v*res';
zk=zk+hz(j+1,LAM)*v*res';
j=j+1; end;

s1=r*s+zk;


% ======================================================================
% SUBFUNCTION
% THIS IS BASED ON 
%   Tucker McElroy, 2008, 'Exact formulas for the Hodrick-Prescott filter', 
%   Econometrics JOurnal, Vol. 11, pp. 209-217
%     see equations (2.12), (2.14), (2.15) and (3.4)

function zar = hz(j,LAM);
j=abs(j);
q=1/LAM;
c=16*q/(sqrt(q)+sqrt(q+16)+sqrt(2*q+2*sqrt(q)*sqrt(q+16)))^2;
     Z=(sqrt(q)+sqrt(q+16)+sqrt(2*q+2*sqrt(q)*sqrt(q+16)))/4;
                   TH=atan(sqrt(2*q+2*sqrt(q)*sqrt(q+16))/4);

zar=0^j-2*c*(Z^(4-j))*sin(TH)*(Z^2*sin((j+1)*TH)-sin((j-1)*TH))/ ...
  ((1-2*cos(2*TH)*Z^2+Z^4)*(Z^2-1)*(1-cos(2*TH)));

%  if LAM=1600: 
%  if j==0; zar=1-(0.056168); end;
%  if j ~= 0; zar=-(0.8941^j)*(0.056168*cos(.11168*j)+.055833*sin(.11168*j));
%  end;
